/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "jabber.h"
#include "jabprivate.h"
#include "jabstream.h"

_jabcbe_on _jabcb_on=NULL;
_jabcbe_off _jabcb_off=NULL;
_jabcbe_msg _jabcb_msg=NULL;
_jabcbe_prs _jabcb_prs=NULL;
_jabcbe_iq _jabcb_iq=NULL;
_jabcbe_auth _jabcb_auth=NULL;
_jabcbe_roster _jabcb_roster=NULL;
_jabcbe_profile _jabcb_profile=NULL;


jabc jab_connect(char *user, char *pass, int flags)
{
    pool p;
    jabc j;

    p = pool_new();
    if(!p) return(NULL);
    j = (jabc)pmalloc(p,sizeof(_jabc));
    if(!j) return(NULL);
    memset(j,0,sizeof(_jabc));
    j->p = p;

    j->state = JAB_S_OFF;
    j->user = jid_new(p,user);
    j->pass = pstrdup(p,pass);
    j->flags = flags;

    if(j->flags&JABC_FASTIO) {
	/* good place to add nonblocking connect */
	return(NULL);
    } else {
	j->fd = make_netsocket(5222,j->user->server,NETSOCKET_CLIENT);
	if (j->fd < 0) return(NULL);
	jabstream_start(j,jabstream_header("jabber:client",j->user->server,NULL));
	j->state = JAB_S_STREAM;
    }

    return(j);
}


void jab_disconnect(jabc j)
{
    if(!j) return;
    jabstream_stop(j,1,NULL);
    pool_free(j->p);
}


int jab_fd(jabc j)
{
    return(j->fd);
}


int jab_isonline(jabc j)
{
    if (JAB_S_ON != j->state) return(0);
    return(1);
}


void jab_setcb(int cb, jabEvent func)
{
    switch(cb) {
	case JABCB_ONLINE: _jabcb_on=(_jabcbe_on)func; break;
	case JABCB_MESSAGE: _jabcb_msg=(_jabcbe_msg)func; break;
	case JABCB_PRESENCE: _jabcb_prs=(_jabcbe_prs)func; break;
	case JABCB_IQ: _jabcb_iq=(_jabcbe_iq)func; break;
	case JABCB_ROSTER: _jabcb_roster=(_jabcbe_roster)func; break;
	case JABCB_PROFILE: _jabcb_profile=(_jabcbe_profile)func; break;
	case JABCB_AUTH: _jabcb_auth=(_jabcbe_auth)func; break;
	case JABCB_OFFLINE: _jabcb_off=(_jabcbe_off)func; break;
    }
}


void jab_poll(jabc j)
{
    fd_set fds;
    struct timeval tv;
    int s;

    if (!j) return;
    if (JAB_S_OFF == j->state) return;

    s = j->fd;
    FD_ZERO(&fds);
    FD_SET(s,&fds);
    tv.tv_sec = 0;
    tv.tv_usec = 25;
    if (select(s+1,&fds,NULL,NULL,&tv) <= 0) return;

    jabstream_read(j);
}


/*
** private functions
*/

void _jab_parsenode(jabc j, xmlnode x, int type)
{
    jabn node;

    switch(type) {
	case JABSTREAM_ROOT:
	    j->state = JAB_S_ON;
	    if(_jabcb_on) (_jabcb_on)(j);
	    if (j->flags&JABC_NEWUSER)
		_jab_reg(j);
	    else
		_jab_auth(j);
	    break;
	case JABSTREAM_NODE:
	    node = jab_parsenode(x);
	    if(!node) break;
	    if(!node->jn.to) node->jn.to = j->user;
	    if (node->jn.node==JABN_MSG && _jabcb_msg) {
		(_jabcb_msg)(j,(jabm)node);
		/* we don't free msg nodes, this is client's job */
	    } else if(node->jn.node==JABN_PRS && _jabcb_prs) {
		(_jabcb_prs)(j,(jabp)node);
		jab_freenode(node);
	    } else if(node->jn.node==JABN_IQ) {
		if(_jabcb_auth && j_strcmp(node->jq.xmlns,"jabber:iq:auth")==0) {
		    _jab_parseauth(j,(jabq)node,0);
		} else if(_jabcb_auth && j_strcmp(node->jq.xmlns,"jabber:iq:register")==0) {
		    _jab_parseauth(j,(jabq)node,1);
		} else if(_jabcb_roster && j_strcmp(node->jq.xmlns,"jabber:iq:roster")==0) {
		    _jabr_parse(j,(jabq)node);
		} else if(_jabcb_profile && j_strcmp(node->jq.xmlns,"jabber:iq:info")==0) {
		    _jabpf_parse(j,(jabq)node);
		} else if(_jabcb_iq) {
		    (_jabcb_iq)(j,(jabq)node);
		}
		jab_freenode(node);
	    }
	    break;
	case JABSTREAM_CLOSE:
	    jabstream_stop(j,0,NULL);
	    if (_jabcb_off) (_jabcb_off)(j,JAB_OFF_BYE);
	    break;
	case JABSTREAM_XMLERR:
	    jabstream_stop(j,1,"Bad XML");
	    if (_jabcb_off) (_jabcb_off)(j,JAB_OFF_XML);
	    break;
	case JABSTREAM_SOCKERR:
	    jabstream_stop(j,0,NULL);
	    if (_jabcb_off) (_jabcb_off)(j,JAB_OFF_SOCK);
	    break;
    }
}
