/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "jabber.h"
#include "jabprivate.h"

void jabpf_ask(jabc j, char *user)
{
    jabn node;
    xmlnode xml;

    xml = xmlnode_new_tag("query");
    xmlnode_put_attrib(xml,"xmlns","jabber:iq:info");

    xmlnode_insert_tag(xml,"email");
    xmlnode_insert_tag(xml,"name");
    xmlnode_insert_tag(xml,"description");
    xmlnode_insert_tag(xml,"email");
    xmlnode_insert_tag(xml,"url");

    node = jab_newnode(JABN_IQ,
			JABQ_TO, user,
			JABQ_TYPE, JABQ_TYPE_GET,
			JABQ_QUERY, xml,
			JABQ_DONE);
    jab_sendnode(j,node);
    xmlnode_free(xml);
}


void jabpf_set(jabc j, int element, char *value)
{
    pool p = j->p;
    jabpf pf = j->profile;

    if(!pf) {
	pf = (jabpf)pmalloc(p,sizeof(_jabpf));
	memset(pf,0,sizeof(_jabpf));
	j->profile = pf;
    }

    switch(element) {
	case JABPF_TYPE: pf->type=pstrdup(p,value); break;
	case JABPF_NAME: pf->name=pstrdup(p,value); break;
	case JABPF_DESCRIPTION: pf->description=pstrdup(p,value); break;
	case JABPF_EMAIL: pf->email=pstrdup(p,value); break;
	case JABPF_URL: pf->url=pstrdup(p,value); break;
    }
}


void jabpf_free(jabpf pf)
{
    pool_free(pf->p);
}


void _jabpf_parse(jabc j, jabq iq)
{
    if(iq->type==JABQ_TYPE_GET && j->profile) {
	jabn node;
	xmlnode xml,tmp;

        xml = xmlnode_new_tag("query");
        xmlnode_put_attrib(xml,"xmlns","jabber:iq:info");

        if(xmlnode_get_tag(iq->query,"type") && j->profile->type) {
    	    tmp = xmlnode_insert_tag(xml,"type");
	    xmlnode_insert_cdata(tmp,j->profile->type,-1);
        }
        if(xmlnode_get_tag(iq->query,"name") && j->profile->name) {
    	    tmp = xmlnode_insert_tag(xml,"name");
	    xmlnode_insert_cdata(tmp,j->profile->name,-1);
        }
        if(xmlnode_get_tag(iq->query,"description") && j->profile->description) {
    	    tmp = xmlnode_insert_tag(xml,"description");
	    xmlnode_insert_cdata(tmp,j->profile->description,-1);
        }
        if(xmlnode_get_tag(iq->query,"email") && j->profile->email) {
    	    tmp = xmlnode_insert_tag(xml,"email");
	    xmlnode_insert_cdata(tmp,j->profile->email,-1);
        }
        if(xmlnode_get_tag(iq->query,"url") && j->profile->url) {
    	    tmp = xmlnode_insert_tag(xml,"url");
	    xmlnode_insert_cdata(tmp,j->profile->url,-1);
        }

	node = jab_newnode(JABN_IQ,
			    JABQ_TYPE, JABQ_TYPE_RESULT,
			    JABQ_TO, jid_full(iq->from),
			    JABQ_QUERY, xml,
			    JABQ_DONE);
	jab_sendnode(j,node);
	xmlnode_free(xml);

    } else if(iq->type == JABQ_TYPE_RESULT) {
	jabpf pf;
	pool p;
	xmlnode tmp;

	p = pool_new();
	pf = (jabpf)pmalloc(p,sizeof(_jabpf));
	memset(pf,0,sizeof(_jabpf));
	pf->p = p;
	if(iq->from)
	    pf->from = pstrdup(p,jid_full(iq->from));
	else
	    j->profile = pf;

	tmp = xmlnode_get_tag(iq->query,"type");
	if(tmp) pf->type=pstrdup(p,xmlnode_get_data(xmlnode_get_firstchild(tmp)));
	tmp = xmlnode_get_tag(iq->query,"name");
	if(tmp) pf->name=pstrdup(p,xmlnode_get_data(xmlnode_get_firstchild(tmp)));
	tmp = xmlnode_get_tag(iq->query,"description");
	if(tmp) pf->description=pstrdup(p,xmlnode_get_data(xmlnode_get_firstchild(tmp)));
	tmp = xmlnode_get_tag(iq->query,"email");
	if(tmp) pf->email=pstrdup(p,xmlnode_get_data(xmlnode_get_firstchild(tmp)));
	tmp = xmlnode_get_tag(iq->query,"url");
	if(tmp) pf->url=pstrdup(p,xmlnode_get_data(xmlnode_get_firstchild(tmp)));

	(_jabcb_profile)(j,pf);
    }
}
